/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.controllers;

import tp.model.DocsRepresentDB;
import tp.view.TabResultsView;

/**
 * A controller to process events on a screen, where the statistical results are presented.
 * @author Ondrej Sabatka
 */

public class ResultsControl extends ActionControl {

    //Clenske promenne tridy
    DocsRepresentDB documents_representation;


    //Constructor
    public ResultsControl() {}

    /**
     * If the "Next" button is clicked, this method executes the tab of results saving
     * and passes document representations to the controller of saving.
     */
    public void nextBtnClick() {
        MainControl.getInstance().gotoSave(documents_representation);
    }


    /**
     * Initializes the results window tab and shows all statistical information
     * about pre-processing.
     * @param representation of all documents
     */
    public void init(DocsRepresentDB dr) {
        this.documents_representation = dr;
        
        //Reference to view
        TabResultsView view = MainControl.getInstance().getMainView().getTabView().getTabResults();

        //Settings of view parameteres
        if(dr.getPreprocessing() == DocsRepresentDB.Preprocessing.STOP_LIST) {
            view.setStopList(true);
            view.setStemming(false);
        }
        else if(dr.getPreprocessing() == DocsRepresentDB.Preprocessing.STEMMING) {
            view.setStemming(true);
            view.setStopList(false);
        }
        else if(dr.getPreprocessing() == DocsRepresentDB.Preprocessing.STOP_LIST_STEMMING) {
            view.setStemming(true);
            view.setStopList(true);
        }
        else {
            view.setStemming(false);
            view.setStopList(false);
        }


        //Setting of feature
        view.setFeature(dr.getFeature(), dr.getNGramDepth(), dr.getMinNGramOccur());

        //Setting of model
        view.setModel(dr.getModel());


        //SETTING OF STATISTICAL RESULTS
        view.setCountDocuments(dr.getCountDocuments());
        view.setCountFeatures(dr.getCountFeatures());
        view.setAvgCountFeatures(dr.getAvgCountFeatures());
        view.setMaxCountFeatures(dr.getMaxCountFeatures());

        if(dr.getModel() != DocsRepresentDB.RepresentationModel.BINARY) {
            view.setMaxFrequentFeature(dr.getMaxFrequentFeature());
            view.setAvgFrequentFeature(dr.getAvgFrequentFeature());
        }
        
        if(dr.getModel() == DocsRepresentDB.RepresentationModel.TF) {
            view.setMaxTFFeature(dr.getMaxTFFeature());
            view.setAvgTFFeature(dr.getAvgTFFeature());
        }
        else if(dr.getModel() == DocsRepresentDB.RepresentationModel.TFIDF) {
            view.setMaxTFidfFeature(dr.getMaxTFidfFeature());
            view.setAvgTFidfFeature(dr.getAvgTFidfFeature());
        }
    }

}
