/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.controllers;

import java.awt.Cursor;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import tp.model.DocsRepresentDB;
import javax.swing.filechooser.FileFilter;
import tp.store.StoreCSV;
import tp.store.StoreInterface;
import tp.view.ProgressView;

/**
 * A controller to process events on a screen, where the saving of results is executed.
 * @author Ondrej Sabatka
 */

public class SaveControl extends ActionControl {

    //Clenske promenne tridy
    private DocsRepresentDB documents_representation;
    private boolean write_topic = true;
    private int restrict_min_occur, restrict_max_occur;

    private ArrayList<DocsRepresentDB.FeatureDF> features_array;


    //Output file formats
    public static final String[] outpuFileFormatsName = {"CSV"};  

    public static final String[] csvSeparatorName = {"Comma", "Semicolon"};
    public static final int COMMA = 0;
    public static final int SEMICOLON = 1;
    private int separator = COMMA;

    //Position of document format
    public static final int CSV = 0;

    private int file_format;

    //Used to save the results
    private final JFileChooser fc = new JFileChooser();
    private File fc_file;

    private FileFilter ff = new FileFilter() {
        @Override
        public boolean accept(File f) {
            if(f.isDirectory()) {
                return true;
            }

            String s = f.getName();

            if((s.indexOf(".csv") > 0) ||
               (s.indexOf(".CSV") > 0))
            {
                return true;
            }

            return false;
        }

        public String getDescription() {
            return "*.CSV";
        }
    };


    //Constructor
    public SaveControl() {
        this.fc.setFileFilter(ff);
    }


    /**
     * Method is called, if a Save button is clicked. It saves all document representation
     * into a CSV file.
     */
    public void saveBtnClick() {
        int returnVal = this.fc.showSaveDialog(MainControl.getInstance().getMainView());

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            fc_file = fc.getSelectedFile();

            //Test, if user entered the file suffix
            if(fc_file.getName().indexOf(".csv") < 0) {
                //If not, a file with the suffix will be created
                fc_file = new File(fc_file.getPath() + ".csv");
            }

            if(fc_file.exists()) {
                //A question, if user wants to overwrite existing file.
                Object[] options = {"Yes",
                                    "No"};

                int n = JOptionPane.showOptionDialog(MainControl.getInstance().getMainView(),
                            "Do you wish to overwrite " + this.fc.getSelectedFile().getName() + " ?",
                            "Confirmation",
                            JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE,
                            null,
                            options,
                            options[0]);

                if(n != 0) {
                    return;
                }
            }


            //The task is executed.
            Task task = new Task();
            task.execute();
        }
    }


    /**
     * A class, which ensures execution of file saving process in the background.
     */
    class Task extends SwingWorker<Void, Void> {

        /**
         * A method which initializes and executes file saving in the background.
         */
        @Override
        public Void doInBackground() {
             //Cursor is set to wait and progress bar is initialized
            MainControl.getInstance().getMainView().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            ProgressView progress = MainControl.getInstance().getMainView().getProgressBar();
            
            progress.start("Saving into a file");
            this.addPropertyChangeListener(progress);


            //Gets the file names.
            File file = fc_file;

            //Removal of too frequent and too infrequent features
            ArrayList<DocsRepresentDB.FeatureDF> restrict_features_array = new ArrayList<DocsRepresentDB.FeatureDF>();

            for(int i = 0; i < features_array.size(); i++) {
                DocsRepresentDB.FeatureDF f = features_array.get(i);

                if((f.document_frequent >= restrict_min_occur) &&
                   (f.document_frequent <= restrict_max_occur)) {
                    restrict_features_array.add(f);
                }
            }

            //Saving into a file
            StoreInterface store;

            switch(file_format) {
                default:
                case CSV:
                    if(separator == COMMA) {
                        store = new StoreCSV(",");
                    }
                    else {
                        store = new StoreCSV(";");
                    }
                    break;
            }

            //Setting of sending information to progress bar
            store.addPropertyChangeListener(progress);

            //Call of method, which saves data into a file
            store.writeDocRepresentation(file, restrict_features_array, documents_representation, write_topic);

            return null;
        }


        /**
         * This called after finishing of data saving. It changes setting of
         * progress bar and cursor.
         */
        @Override
        public void done() {
            MainControl.getInstance().getMainView().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            MainControl.getInstance().getMainView().getProgressBar().stop();
        }
    }
    

    /**
     * After click on "Quit" button, application is halted.
     */
    public void endBtnClick() {
        System.exit(0);
    }

    /**
     * Method used to change the separator of values in CSV files.
     */
    public void changeSeparator(String s) {
        this.separator = Integer.parseInt(s);
    }

    /**
     * Method used to get default separator of values in CSV files.
     * @return default separator
     */
    public int getDefaultSeparator() {
        return this.separator;
    }
    

    /**
     * Initialization of a tab for saving the pre-processed documents. Some limits of
     * data saving are computed.
     * @param database of document representations
     */
    public void init(DocsRepresentDB dr) {
        this.documents_representation = dr;
        
        int min_min_restrict = 0;
        int min_max_restrict = (int) (dr.getCountDocuments() * 0.1);  //Max low limit = 10%
        int max_min_restrict = (int) (dr.getCountDocuments() * 0.6);  //Max high limit = 40%
        int max_max_restrict = dr.getCountDocuments();

        restrict_min_occur = (int) ((min_max_restrict - min_min_restrict) / 15);
        restrict_max_occur = max_min_restrict + (int) ((max_max_restrict - max_min_restrict) / 2);

        //Setting of limits according to the dataset
        MainControl.getInstance().getMainView().getTabView().getTabSave().
                setRestrictLimits(min_min_restrict, min_max_restrict, restrict_min_occur,
                                  max_min_restrict, max_max_restrict, restrict_max_occur);


        //Get all features from document
        features_array = documents_representation.getFeaturesDocumentFreq();

        //Sorting of features by occurence
        Collections.sort(features_array);

        actualizeCountFeatures();
    }

    /**
     * Method used to get information, if the category of documents should be stored.
     * @return boolen value, "1" if category should be stored
     */
    public boolean writeCategory() {
        return write_topic;
    }

    /**
     * Method used to change information, if the category of documents should be stored.
     */
    public void writeCategoryChange() {
        this.write_topic = !this.write_topic;
    }

    /**
     * Method used to actualize count features for the whole dataset.
     */
    private void actualizeCountFeatures() {
        int count = 0;

        for(int i = 0; i < features_array.size(); i++) {
            DocsRepresentDB.FeatureDF f = features_array.get(i);

            if((f.document_frequent >= restrict_min_occur) &&
               (f.document_frequent <= restrict_max_occur)) {
                count++;
            }
        }

        MainControl.getInstance().getMainView().getTabView().getTabSave().setActualCountFeatures(count);
    }


    /**
     * Method used to change actual setting of restriction of minimum occurence of a feature.
     */
    public void setRestrictMinOccur() {
        this.restrict_min_occur =
                MainControl.getInstance().getMainView().getTabView().getTabSave().getRestrictMinOccur();

        actualizeCountFeatures();
    }


    /**
     * Method used to change actual setting of restriction of maximum occurence of a feature.
     */
    public void setRestrictMaxOccur() {
        this.restrict_max_occur =
                MainControl.getInstance().getMainView().getTabView().getTabSave().getRestrictMaxOccur();
        
        actualizeCountFeatures();
    }

    /**
     * Method used to change actual setting of file format.
     */
    public void changeFileFormat() {
        this.file_format =
                MainControl.getInstance().getMainView().getTabView().getTabSave().getFileFormat();
    }

}
