/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.model;

/**
 * Class representing a feature and its frequency in TF or TFIDF model.
 * @author Ondrej Sabatka
 */

public class FeatureFrequency implements FeatureInterface {

    //Variables
    private String feature;
    private int frequency;
    private float tf;
    private float tfidf;


    /**
     * Constructor of the class
     * @param feature a feature (word)
     * @param frequency a value of TF or TFIDF weight
     */
    public FeatureFrequency(String feature, int frequency) {
        this.feature = feature;
        this.frequency = frequency;
        this.tfidf = 0;
    }

    //**************************************************************//
    //                      GETTERS AND SETTERS                     //
    //**************************************************************//
    /**
     * Gets a word, which is represented by this feature.
     * @return a feature (word)
     */
    public String getFeature() {
        return feature;
    }

    /**
     * Gets a binary frequency of this feature.
     * @return value of frequency (0 or 1)
     */
    public int getRate() {
        return frequency;
    }

    /**
     * Sets a word, which is represented by this feature.
     * @param feature a feature (word)
     */
    public void setFeature(String feature) {
        this.feature = feature;
    }

    /**
     * Sets a binary frequency of this feature.
     * @param rate value of frequency (0 or 1)
     */
    public void setRate(int rate) {
        this.frequency = rate;
    }

    /**
     * Gets a TF-IDF weight of this feature.
     * @return value of TF-IDF weight
     */
    public float getTFidf() {
        return tfidf;
    }

    /**
     * Sets a TF-IDF weight for this feature.
     * @param tfidf value of TF-IDF weight
     */
    public void setTFidf(float tfidf) {
        this.tfidf = tfidf;
    }

    /**
     * Gets a TF weight of this feature.
     * @return value of TF weight
     */
    public float getTF() {
        return tf;
    }

    /**
     * Sets a TF weight for this feature.
     * @param tf value of TF weight
     */
    public void setTF(float tf) {
        this.tf = tf;
    }

    
}
