/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.model;

import java.io.File;
import java.util.Vector;

/**
 * A class representing links to files, which are chosen by the user for pre-processing.
 * @author Ondrej Sabatka
 */

public class LinkToFiles {

    //Variables

    //File types
    public enum FileType {PLAIN_TEXT, REUTERS}
    private FileType file_type;

    //Array of files
    private Vector<File> files;

    //Cursor to array of files
    private int cursor;


    /**
     * Constructor of the class
     */
    public LinkToFiles() {
        //Initialization of variables
        file_type = null;
        files = null;
        //Not used, value will be -1
        cursor = -1;
    }


    /**
     * Method initializes vecotr of files a stores them
     * @param new_files array of file to process
     * @return number of wrongly loaded files
     */
    public int addFiles(File[] new_files) {
        int count_wrong = 0;

        if(files == null) {
            files = new Vector<File>();
        }

        for(int i = 0; i < new_files.length; i++) {
            if(!files.contains(new_files[i])) {
                files.add(new_files[i]);
            }
            else {
                count_wrong++;
            }
        }

        return count_wrong;
    }


    /**
     * Method consequently returns items from array of files
     * @return next item, which has not been returned yet
     */
    public File getFileSingly() {
        if(files == null) {
            return null;
        }

        //1st usage of cursor
        if(cursor <= -1) {
            cursor = 0; 
            return files.get(cursor++);
        }
        //consequently returns items, until the end of array
        else if(cursor < files.capacity()) {
            return files.get(cursor++);
        }
        else {
            cursor = -1;
            return null;
        }
    }


    /**
     * Returns a file from array being on a given item index
     * @param index index of file
     * @return chosen file
     */
    public File getFileAt(int index) {
        return files.get(index);
    }


   /**
     * Removes a file from array being on a given item index
     * @param i index of file
     */
    public void removeFile(int i) {
        files.remove(i);
    }


    /**
     * Removes files from array being on a given item indexes
     * @param index array of indexes
     */
    public void removeFiles(int[] index) {
        for(int i = (index.length - 1); i >= 0 ; i--) {
            files.remove(index[i]);
        }
    }


    /**
     * Sets used file type.
     * @param type file type
     */
    public void setFileType(FileType type) {
        file_type = type;
    }


    /**
     * Gets actual value of file type.
     * @return file type
     */
    public FileType getFileType() {
        return file_type;
    }


    /**
     * Gets count of items in the array.
     * @return count of items
     */
    public int getCount() {
        return files.size();
    }


    /**
     * Gets information, if the array is empty or not.
     * @return true, if it is empty, false otherwise
     */
    public boolean isEmpty() {
        if((files == null) || (files.size() == 0)) {
            return true;
        }
        else {
            return false;
        }
    }


}
