/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.model;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import javax.swing.ImageIcon;
import tp.Main;

/**
 * Class, which secures access to files needed by the application.
 * @author Ondrej Sabatka
 */

public class ResourcesModel {

    /**
     * Constructor of the class
     */
    public ResourcesModel() {}



    /**
     * Returns an image icon from given file.
     * @param path path to a file containing an image icon
     * @return the image icon
     */
    public ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = Main.class.getResource("resources/" + path);
        return new ImageIcon(imgURL);
    }


    /**
     * Returns an HTML code of a page, which should be represented in the help panel.
     * @param file_name file name of the corresponding HTML page
     * @return a string with HTML code
     */
    public String getHelpPage(String file_name) {
        URL page_url = Main.class.getResource("resources/help/" + file_name);
        return page_url.toString();
    }


    /**
     * Returns a path to a file, which contains actual stop-list.
     * @return a file with a stop-list
     */
    public BufferedReader getStopListPath() {
        InputStream is = Main.class.getResourceAsStream("resources/stop-list/stop-list.csv");
        return new BufferedReader(new InputStreamReader(is));
    }
}
