/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import tp.controllers.MainControl;

/**
 * Class responsible for displaying pagen on the help panel.
 * @author Ondrej Sabatka
 */

public class HelpView extends JEditorPane {

    //Clenske promenne tridy
    private static final int VERTICAL_INSET = 5;
    private static final int HORIZONTAL_INSET = 10;
    private String actual_shown_help;


    /**
     * Constructor of the class
     */
    public HelpView() {
        //Setting of a gap between the text and edge of the window
        setBorder(BorderFactory.createEmptyBorder(VERTICAL_INSET, HORIZONTAL_INSET,
                                                  VERTICAL_INSET, HORIZONTAL_INSET)); //top, left, bottom, right

        //Turn off the editability
        setEditable(false);

        //If a mouse appears on the help panel, the main help page is displayed
        this.setName("help-intro.html");
        this.addMouseListener(MainControl.getInstance().getHelpControl());
    }


    /**
     * Reads the help file from the tp.recources.help package
     * @param file_path path to a help file
     */
    public void showPage(String file_path) {
        if((file_path != null) || !actual_shown_help.equals(file_path)) {
            try {
                //Displays actual help page on the panel
                setPage(file_path);

                //Saves information about actually displayed page
                actual_shown_help = file_path;
            } catch (IOException e) {

                //If it is not successful, the main page is displayed again
                try {
                    setPage(actual_shown_help);
                } catch (IOException ex) {}
            }
        }
    }


    /**
     * Returns the name of currently displayed help page.
     * @return name of the help page, which is displayed
     */
    public String getActualShownHelp() {
        return actual_shown_help;
    }

}
