/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import tp.controllers.MainControl;

/**
 * Class for the main application window.
 * @author Ondrej Sabatka
 */

public class MainView extends JFrame {

    //Variables and constants
    private static final int WINDOW_WIDTH = 900;
    private static final int WINDOW_HEIGHT = 550;
    private MenuView menu_bar;
    private ProgressView progress;
    private HelpView help_view;
    private TabView tabs;

    /**
     * Constructor of the main window.
     */
    public MainView() {
        //Implicit operation to exit
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);

        //Basic parameters of the window
        initWindow();

        //Heading of the window
        this.setTitle("Pre-processing of text documents for text mining");

        //Window contents initialization (menu, tabs, help, status bar)
        initWindowContent();

        //Reading of main help page
        help_view.showPage(MainControl.getInstance().getResourceModel().getHelpPage("help-intro.html"));

        //Displaying the window
        this.setVisible(true);
    }



    /**
     * Initializaton of basic parameters of main window.
     */
    private void initWindow() {
        //Size of the window
        Dimension window_dimewnsion = new Dimension(WINDOW_WIDTH, WINDOW_HEIGHT);
        this.setMinimumSize(window_dimewnsion);

        //Position of the window -> centre of the screen
        this.setLocationRelativeTo(null);
    }


    /**
     * Main window contents initialization
     */
    private void initWindowContent() {
        //Main panel for placing components
        JPanel content = new JPanel();

        // 1) Menu
        menu_bar = new MenuView();
        this.setJMenuBar(menu_bar);


        ////////////////////////////////////////////////////////
        // Divided into two panels - upper for tabs and help  //
        //                         - bottom for progress bar  //
        ////////////////////////////////////////////////////////

        //Layout setting
        GridBagConstraints area = new GridBagConstraints();
        content.setLayout(new GridBagLayout());
        area.fill = GridBagConstraints.BOTH;

        //Upper panel
        JPanel top_panel = new JPanel(new GridLayout(1, 2));
        top_panel.setBorder(BorderFactory.createLineBorder(Color.black));

        area.gridx = 0;
        area.gridy = 0;
        area.weightx = 0.5;
        area.weighty = 0.99;

        content.add(top_panel, area);

        //Bottom panel
        JPanel bottom_panel = new JPanel(new GridLayout(1, 1));

        area.gridy = 1;
        area.weighty = 0.01;
        area.fill = GridBagConstraints.HORIZONTAL;

        content.add(bottom_panel, area);


        //Adding of contents
        // 2) Tabs
        tabs = new TabView();
        top_panel.add(tabs);

        
        // 3) Help panel
        help_view = new HelpView();
        JScrollPane scroll_pane = new JScrollPane(help_view);
        top_panel.add(scroll_pane);


        // 4) status bar
        progress = new ProgressView();
        bottom_panel.add(progress);


        //Inserting components to the window
        this.add(content);
    }


    /**
     * Returns a link to help panel.
     * @return link to help panel
     */
    public HelpView getHelpView() {
        return help_view;
    }


    /**
     * Returns a link to the main tab.
     * @return link to the main tab
     */
    public TabView getTabView() {
        return tabs;
    }

    /**
     * Returns a link to the progress bar.
     * @return link to the progress bar
     */
    public ProgressView getProgressBar() {
        return progress;
    }


    /**
     * Displays an error message
     * @param msg text of an error message
     */
    public void showErrorDialog(String msg) {
        JOptionPane.showMessageDialog(this, msg, "Error alert",
                JOptionPane.ERROR_MESSAGE);
    }

}
