/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.util.Formatter;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import tp.controllers.MainControl;
import tp.model.DocsRepresentDB;

/**
 * Class, which is displaying the panel for pre-processing results in the application.
 * @author Ondrej Sabatka
 */

public class TabResultsView extends JPanel {

    //Variables
    private ActionListener action_handler;

    private JLabel count_documents = new JLabel("---");
    private JLabel count_features = new JLabel("---");
    private JLabel max_count_features = new JLabel("---"); 
    private JLabel avg_count_features = new JLabel("---");

    private JLabel max_frequent_feature = new JLabel("---");
    private JLabel avg_frequent_feature = new JLabel("---");
    private JLabel max_tf_feature = new JLabel("---");
    private JLabel avg_tf_feature = new JLabel("---");
    private JLabel max_tfidf_feature = new JLabel("---");
    private JLabel avg_tfidf_feature = new JLabel("---");

    private JLabel stop_list = new JLabel(), stemming = new JLabel();
    private JLabel features = new JLabel();
    private JLabel model = new JLabel();

/**
 * Constructor of the class.
 */
    public TabResultsView() {
        //Controller setting to process events from GUI
        action_handler = MainControl.getInstance().getResultsControl();

        //Layout setting
        GridBagConstraints area = new GridBagConstraints();
        this.setLayout(new GridBagLayout());
        area.fill = GridBagConstraints.BOTH;

        //Fonts
        GUIFonts font = new GUIFonts();

        ////////////////////////////////////////////////////////////////////////
        //                     PRE-PROCESSING OPTIONS PANEL                   //
        ////////////////////////////////////////////////////////////////////////
        JPanel settings_panel = new JPanel(new GridBagLayout());
        GridBagConstraints settings_area = new GridBagConstraints();
        
        settings_panel.setBorder(BorderFactory.createTitledBorder("Setting of pre-processing and representation"));

        //Help about format setting...
        settings_panel.setName("results.html");
        settings_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridx = 0;
        area.gridy = 0;
        area.weightx = 0.5;
        area.weighty = 0.2;

        this.add(settings_panel, area);

        //Pre-processing
        settings_area.gridx = 0;
        settings_area.gridy = 0;
        settings_area.weightx = 0.1;
        settings_area.weighty = 0.1;
        settings_area.anchor = GridBagConstraints.WEST;

        JLabel label = new JLabel("Pre-processing: ");
        label.setForeground(Color.GRAY);
        settings_panel.add(label, settings_area);

        settings_area.gridx = 1;
        label = new JLabel("stop-list: ");
        label.setFont(font.getText());
        settings_panel.add(label, settings_area);

        settings_area.gridx = 2;
        settings_panel.add(stop_list, settings_area);

        settings_area.gridx = 3;
        label = new JLabel(" stemming: ");
        label.setFont(font.getText());
        settings_panel.add(label, settings_area);

        settings_area.gridx = 4;
        settings_panel.add(stemming, settings_area);


        //Featrues settings...
        settings_area.gridy = 1;
        settings_area.gridx = 0;
        label = new JLabel("Choose features: ");
        label.setForeground(Color.GRAY);
        settings_panel.add(label, settings_area);

        settings_area.gridx = 1;
        settings_area.gridwidth = 5;
        settings_panel.add(features, settings_area);

        //Model settings...
        settings_area.gridy = 2;
        settings_area.gridwidth = 1;
        settings_area.gridx = 0;
        label = new JLabel("Choose model: ");
        label.setForeground(Color.GRAY);
        settings_panel.add(label, settings_area);

        settings_area.gridx = 1;
        settings_area.gridwidth = 5;
        settings_panel.add(model, settings_area);

        

        

        ////////////////////////////////////////////////////////////////////////
        //                           STATISTICS PANEL                         //
        ////////////////////////////////////////////////////////////////////////
        JPanel statistics_panel = new JPanel(new GridBagLayout());
        GridBagConstraints statistics_area = new GridBagConstraints();

        statistics_panel.setBorder(BorderFactory.createTitledBorder("Statistics"));

        //Setting of help for rormat
        statistics_panel.setName("results.html");
        statistics_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridy = 1;
        area.weighty = 0.7;

        this.add(statistics_panel, area);
        
        statistics_area.gridx = 0;
        statistics_area.gridy = 0;
        statistics_area.weightx = 0.3;
        statistics_area.weighty = 0.3;
        statistics_area.anchor = GridBagConstraints.WEST;

        label = new JLabel("Count of documents: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(count_documents, statistics_area);


        statistics_area.gridy = 1;
        statistics_area.gridx = 0;
        label = new JLabel("Count of features: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(count_features, statistics_area);


        statistics_area.gridy = 2;
        statistics_area.gridx = 0;
        label = new JLabel("Max. number of features: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(max_count_features, statistics_area);

        statistics_area.gridx = 2;
        label = new JLabel(" in a document");
        label.setFont(font.getText());
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);


        statistics_area.gridy = 3;
        statistics_area.gridx = 0;
        label = new JLabel("Avg. number of features: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(avg_count_features, statistics_area);

        statistics_area.gridx = 2;
        label = new JLabel(" in a document");
        label.setFont(font.getText());
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);



        statistics_area.gridy = 4;
        statistics_area.gridx = 0;
        label = new JLabel("Max. frequency of a feature: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(max_frequent_feature, statistics_area);


        statistics_area.gridy = 5;
        statistics_area.gridx = 0;
        label = new JLabel("Avg. frequency of a feature: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(avg_frequent_feature, statistics_area);


        statistics_area.gridy = 6;
        statistics_area.gridx = 0;
        label = new JLabel("Max. TF of a feature: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(max_tf_feature, statistics_area);


        statistics_area.gridy = 7;
        statistics_area.gridx = 0;
        label = new JLabel("Avg. TF of a feature: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(avg_tf_feature, statistics_area);


        statistics_area.gridy = 8;
        statistics_area.gridx = 0;
        label = new JLabel("Max. TF-IDF of a feature: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(max_tfidf_feature, statistics_area);


        statistics_area.gridy = 9;
        statistics_area.gridx = 0;
        label = new JLabel("Avg. TF-IDF of a feature: ");
        label.setForeground(Color.GRAY);
        statistics_panel.add(label, statistics_area);

        statistics_area.gridx = 1;
        statistics_panel.add(avg_tfidf_feature, statistics_area);



        ////////////////////////////////////////////////////////////////////////
        //                            "NEXT" BUTTON                           //
        ////////////////////////////////////////////////////////////////////////
        ImageIcon icon = MainControl.getInstance().getResourceModel().createImageIcon("next.png");
        JButton next_btn = new JButton("Next ", icon);
        next_btn.setFont(font.getTitleText());

        next_btn.setName("nextBtnClick");
        next_btn.addActionListener(action_handler);

        next_btn.setVerticalTextPosition(AbstractButton.CENTER);
        next_btn.setHorizontalTextPosition(AbstractButton.LEADING);

        area.gridy = 2;
        area.weighty = 0.1;
        area.fill = GridBagConstraints.NONE;

        this.add(next_btn, area);

    }




    /**
     * Sets the text about stemming usage (yes or no).
     * @param is boolean value, true, if stemming is used, false otherwise
     */
    public void setStemming(boolean is) {
        if(is) {
            this.stemming.setForeground(Color.green);
            this.stemming.setText("YES");
        }
        else {
            this.stemming.setForeground(Color.red);
            this.stemming.setText("NO");
        }
    }

    /**
     * Sets the text about stoplist usage (yes or no).
     * @param is boolean value, true, if stoplist is used, false otherwise
     */
    public void setStopList(boolean is) {
        if(is) {
            this.stop_list.setForeground(Color.green);
            this.stop_list.setText("YES");
        }
        else {
            this.stop_list.setForeground(Color.red);
            this.stop_list.setText("NO");
        }
    }

    /**
     * Sets the text about feature used to represent text.
     * @param f a feature (words or N-grams)
     * @param depth size of N-gram, if it is used
     * @param min_occur minimum occurence of N-gram, if it is used
     */
    public void setFeature(DocsRepresentDB.RepresentationFeature f, int depth, int min_occur) {
        GUIFonts font = new GUIFonts();
        this.features.setFont(font.getText());
        
        if(f == DocsRepresentDB.RepresentationFeature.NGRAMS) {
            this.features.setText("N-Grams of max. size " + depth + " with min. occurence " + min_occur);
        }
        else {
            this.features.setText("Words");
        }
    }

    /**
     * Sets the text about model used to represent text.
     * @param m a model (binary/TF/TF-IDF)
     */
    public void setModel(DocsRepresentDB.RepresentationModel m) {
        GUIFonts font = new GUIFonts();
        this.model.setFont(font.getText());

        if(m == DocsRepresentDB.RepresentationModel.BINARY) {
            this.model.setText("binary");
        }
        else if(m == DocsRepresentDB.RepresentationModel.TF) {
            this.model.setText("TF (term frequency)");
        }
        else {
            this.model.setText("TF-IDF (term freq. - inverse document freq.)");
        }
    }


    /**
     * Sets the text with value of average count of features.
     * @param avg_count_features a value of average count of features
     */
    public void setAvgCountFeatures(double avg_count_features) {
        Formatter formatter = new Formatter();
        GUIFonts font = new GUIFonts();
        this.avg_count_features.setFont(font.getText());
        this.avg_count_features.setText(formatter.format("%.5f", avg_count_features).toString());
    }

    /**
     * Sets the text with value of average frequency of a feature.
     * @param avg_frequent_feature a value of average frequency of a feature
     */
    public void setAvgFrequentFeature(double avg_frequent_feature) {
        Formatter formatter = new Formatter();
        GUIFonts font = new GUIFonts();
        this.avg_frequent_feature.setFont(font.getText());
        this.avg_frequent_feature.setText(formatter.format("%.5f", avg_frequent_feature).toString());
    }

    /**
     * Sets the text with value of count of features.
     * @param count_features a value of count of features
     */
    public void setCountFeatures(int count_features) {
        GUIFonts font = new GUIFonts();
        this.count_features.setFont(font.getText());
        this.count_features.setText(Integer.toString(count_features));
    }

    /**
     * Sets the text with value of count of documents.
     * @param count_documents a value of count of documents
     */
    public void setCountDocuments(int count_documents) {
        GUIFonts font = new GUIFonts();
        this.count_documents.setFont(font.getText());
        this.count_documents.setText(Integer.toString(count_documents));
    }

    /**
     * Sets the text with value of maximum count of features.
     * @param max_count_features a value of maximum count of features
     */
    public void setMaxCountFeatures(int max_count_features) {
        GUIFonts font = new GUIFonts();
        this.max_count_features.setFont(font.getText());
        this.max_count_features.setText(Integer.toString(max_count_features));
    }

    /**
     * Sets the text with value of maximum frequency of a feature.
     * @param max_frequent_feature a value of maximum frequency of a feature
     */
    public void setMaxFrequentFeature(int max_frequent_feature) {
        GUIFonts font = new GUIFonts();
        this.max_frequent_feature.setFont(font.getText());
        this.max_frequent_feature.setText(Integer.toString(max_frequent_feature));
    }

    /**
     * Sets the text with value of average TF value of a feature.
     * @param avg_tf_feature a value of average TF value of a feature
     */
    public void setAvgTFFeature(double avg_tf_feature) {
        Formatter formatter = new Formatter();
        GUIFonts font = new GUIFonts();
        this.avg_tf_feature.setFont(font.getText());
        this.avg_tf_feature.setText(formatter.format("%.5f", avg_tf_feature).toString());
    }

    /**
     * Sets the text with value of average TF-IDF value of a feature.
     * @param avg_tfidf_feature a value of average TF-IDF value of a feature
     */
    public void setAvgTFidfFeature(double avg_tfidf_feature) {
        Formatter formatter = new Formatter();
        GUIFonts font = new GUIFonts();
        this.avg_tfidf_feature.setFont(font.getText());
        this.avg_tfidf_feature.setText(formatter.format("%.5f", avg_tfidf_feature).toString());
    }

    /**
     * Sets the text with value of maximum TF value of a feature.
     * @param max_tf_feature a value of maximum TF value of a feature
     */
    public void setMaxTFFeature(double max_tf_feature) {
        Formatter formatter = new Formatter();
        GUIFonts font = new GUIFonts();
        this.max_tf_feature.setFont(font.getText());
        this.max_tf_feature.setText(formatter.format("%.5f", max_tf_feature).toString());
    }

    /**
     * Sets the text with value of maximum TF-IDF value of a feature.
     * @param max_tfidf_feature a value of maximum TF-IDF value of a feature
     */
    public void setMaxTFidfFeature(double max_tfidf_feature) {
        Formatter formatter = new Formatter();
        GUIFonts font = new GUIFonts();
        this.max_tfidf_feature.setFont(font.getText());
        this.max_tfidf_feature.setText(formatter.format("%.5f", max_tfidf_feature).toString());
    }

}
