/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import tp.controllers.MainControl;
import tp.controllers.SaveControl;

/**
 * Class, which is displaying the panel for saving the results of the application.
 * @author Ondrej Sabatka
 */

public class TabSaveView extends JPanel {

    //Variables
    private ActionListener action_handler;
    private SaveControl save_controller;
    private JLabel range_min_restrict, range_max_restrict;
    private JLabel actual_count_features;
    JPanel format_csv_panel;

    private JSpinner restrict_spin_min_occur, restrict_spin_max_occur;
    private JComboBox select_format;


/**
 * Constructor of the class.
 */
    public TabSaveView() {
        //Controller setting to process events from GUI
        action_handler = MainControl.getInstance().getSaveControl();
        save_controller = MainControl.getInstance().getSaveControl();

        //Layout settings...
        GridBagConstraints area = new GridBagConstraints();
        this.setLayout(new GridBagLayout());
        area.fill = GridBagConstraints.BOTH;

        //Fonts settings
        GUIFonts font = new GUIFonts();

        ////////////////////////////////////////////////////////////////////////
        //                          SETTINGS PANEL                            //
        ////////////////////////////////////////////////////////////////////////
        JPanel settings_panel = new JPanel(new GridBagLayout());
        GridBagConstraints settings_area = new GridBagConstraints();
        
        settings_panel.setBorder(BorderFactory.createTitledBorder("Options of output"));

        //Help about format setting...
        settings_panel.setName("save_settings.html");
        settings_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridx = 0;
        area.gridy = 0;
        area.weightx = 0.5;
        area.weighty = 0.3;

        this.add(settings_panel, area);

        settings_area.gridx = 0;
        settings_area.gridy = 0;
        settings_area.weightx = 0.5;
        settings_area.weighty = 0.3;
        settings_area.anchor = GridBagConstraints.WEST;


        settings_area.gridwidth = 3;
        JLabel label = new JLabel("Save categories");
        label.setForeground(Color.gray);
        settings_panel.add(label, settings_area);


        settings_area.gridy = 1;
        JCheckBox category_check = new JCheckBox("Category - saved for each text document");
        category_check.setMnemonic(KeyEvent.VK_K);
        category_check.setSelected(save_controller.writeCategory());
        category_check.setFont(font.getText());
        settings_panel.add(category_check, settings_area);

        //Listener setting
        category_check.setName("writeCategoryChange");
        category_check.addItemListener(save_controller);


        settings_area.gridy = 2;
        settings_panel.add(new JLabel(""), settings_area);


        settings_area.gridy = 4;
        label = new JLabel("Output set constraints");
        label.setForeground(Color.gray);
        settings_panel.add(label, settings_area);


        settings_area.gridwidth = 1;
        settings_area.weightx = 0.1;
        settings_area.gridy = 5;

        label = new JLabel("Min. occurence: ");
        label.setFont(font.getText());
        settings_panel.add(label, settings_area);
        

        settings_area.gridx = 1;
        settings_area.weightx = 0.5;
        settings_area.fill = GridBagConstraints.HORIZONTAL;
        restrict_spin_min_occur = new JSpinner();
        restrict_spin_min_occur.setMinimumSize(new Dimension(70, 20));
        restrict_spin_min_occur.setName("setRestrictMinOccur");
        restrict_spin_min_occur.addChangeListener(save_controller);

        settings_panel.add(restrict_spin_min_occur, settings_area);


        settings_area.fill = GridBagConstraints.NONE;
        settings_area.gridx = 2;
        settings_area.weightx = 0.4;
        label = new JLabel(" (omit features with lower occurence)");
        label.setFont(font.getText());
        settings_panel.add(label, settings_area);


        settings_area.gridy = 6;
        settings_area.gridx = 1;
        settings_area.gridwidth = 3;
        range_min_restrict = new JLabel();
        range_min_restrict.setFont(font.getText());
        range_min_restrict.setForeground(Color.gray);
        settings_panel.add(range_min_restrict, settings_area);



        settings_area.gridwidth = 1;
        settings_area.weightx = 0.1;
        settings_area.gridy = 7;
        settings_area.gridx = 0;

        label = new JLabel("Max. occurence: ");
        label.setFont(font.getText());
        settings_panel.add(label, settings_area);

        settings_area.fill = GridBagConstraints.HORIZONTAL;
        settings_area.gridx = 1;
        settings_area.weightx = 0.5;
        restrict_spin_max_occur = new JSpinner();
        restrict_spin_max_occur.setMinimumSize(new Dimension(70, 20));
        restrict_spin_max_occur.setName("setRestrictMaxOccur");
        restrict_spin_max_occur.addChangeListener(save_controller);

        settings_panel.add(restrict_spin_max_occur, settings_area);


        settings_area.fill = GridBagConstraints.NONE;
        settings_area.gridx = 2;
        settings_area.weightx = 0.4;
        label = new JLabel(" (omit features with higher occurence)");
        label.setFont(font.getText());
        settings_panel.add(label, settings_area);


        settings_area.gridy = 8;
        settings_area.gridx = 1;
        settings_area.gridwidth = 3;
        range_max_restrict = new JLabel();
        range_max_restrict.setFont(font.getText());
        range_max_restrict.setForeground(Color.gray);
        settings_panel.add(range_max_restrict, settings_area);


        settings_area.gridy = 9;
        settings_area.gridx = 0;
        label = new JLabel("Actual count of features to save: ");
        label.setForeground(Color.gray);
        settings_panel.add(label, settings_area);

        settings_area.gridy = 10;
        settings_area.gridx = 1;
        actual_count_features = new JLabel();
        actual_count_features.setForeground(Color.red);
        settings_panel.add(actual_count_features, settings_area);

        

        ////////////////////////////////////////////////////////////////////////
        //                               SAVING                               //
        ////////////////////////////////////////////////////////////////////////
        JPanel save_panel = new JPanel(new GridBagLayout());
        GridBagConstraints save_area = new GridBagConstraints();

        save_panel.setBorder(BorderFactory.createTitledBorder("Save"));

        //Setting the help for format
        save_panel.setName("save_format.html");
        save_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridy = 1;
        area.weighty = 0.4;

        this.add(save_panel, area);
        
        save_area.gridx = 0;
        save_area.gridy = 0;
        save_area.weightx = 0.1;
        save_area.weighty = 0.1;
        save_area.anchor = GridBagConstraints.WEST;
        save_area.gridwidth = 3;

        label = new JLabel("File format");
        label.setForeground(Color.gray);
        save_panel.add(label, save_area);


        save_area.gridwidth = 1;
        save_area.gridy = 1;
        //ComboBox to choose the format
        select_format = new JComboBox(SaveControl.outpuFileFormatsName);
        select_format.setFont(font.getBoldText());
        select_format.setMinimumSize(new Dimension(80, 20));
        select_format.setName("changeFileFormat");
        select_format.addActionListener(action_handler);

        save_panel.add(select_format, save_area);

        save_area.weightx = 0.9;
        save_area.gridx = 1;
        label = new JLabel("choose output file format                      ");
        label.setFont(font.getText());
        save_panel.add(label, save_area);


        save_area.gridwidth = 6;
        save_area.gridy = 2;
        save_area.gridx = 0;
        format_csv_panel = new JPanel(new GridLayout(3,1));

        JRadioButton comma = new JRadioButton(SaveControl.csvSeparatorName[SaveControl.COMMA]);
        comma.setMnemonic(KeyEvent.VK_C);
        comma.setFont(font.getText());
        comma.setName("changeSeparator;" + SaveControl.COMMA);
        comma.addActionListener(action_handler);

        JRadioButton semicolon = new JRadioButton(SaveControl.csvSeparatorName[SaveControl.SEMICOLON]);
        semicolon.setMnemonic(KeyEvent.VK_S);
        semicolon.setFont(font.getText());
        semicolon.setName("changeSeparator;" + SaveControl.SEMICOLON);
        semicolon.addActionListener(action_handler);

        ButtonGroup separators = new ButtonGroup();
        separators.add(comma);
        separators.add(semicolon);

        if(save_controller.getDefaultSeparator() == SaveControl.COMMA) {
            comma.setSelected(true);
        }
        else if(save_controller.getDefaultSeparator() == SaveControl.SEMICOLON) {
            semicolon.setSelected(true);
        }


        label = new JLabel("Choose separator for CSV format:");
        label.setFont(font.getText());

        format_csv_panel.add(label);
        format_csv_panel.add(comma);
        format_csv_panel.add(semicolon);

        save_panel.add(format_csv_panel, save_area);


        //SAVE button
        ImageIcon icon = MainControl.getInstance().getResourceModel().createImageIcon("save.png");
        JButton save_btn = new JButton("Save ", icon);
        save_btn.setFont(font.getTitleText());

        save_btn.setName("saveBtnClick");
        save_btn.addActionListener(action_handler);

        save_btn.setVerticalTextPosition(AbstractButton.CENTER);
        save_btn.setHorizontalTextPosition(AbstractButton.LEADING);

        save_area.gridwidth = 3;
        save_area.weighty = 0.7;
        save_area.gridx = 0;
        save_area.gridy = 3;

        save_panel.add(save_btn, save_area);




        ////////////////////////////////////////////////////////////////////////
        //                            QUIT BUTTON                             //
        ////////////////////////////////////////////////////////////////////////
        icon = MainControl.getInstance().getResourceModel().createImageIcon("end.png");
        JButton end_btn = new JButton("Quit ", icon);
        end_btn.setFont(font.getTitleText());

        end_btn.setName("endBtnClick");
        end_btn.addActionListener(action_handler);

        end_btn.setVerticalTextPosition(AbstractButton.CENTER);
        end_btn.setHorizontalTextPosition(AbstractButton.LEADING);

        area.gridy = 2;
        area.weighty = 0.2;
        area.fill = GridBagConstraints.NONE;

        this.add(end_btn, area);

    }

    /**
     * Sets the parameters for the spinners placed on the tab.
     * @param min_min_restrict a minimum restriction value of the first spinner
     * @param min_max_restrict a maximum restriction value of the first spinner
     * @param min_def initial value of the first spinner
     * @param max_min_restrict a minimum restriction value of the second spinner
     * @param max_max_restrict a maximum restriction value of the second spinner
     * @param max_def initial value of the second spinner
     */
    public void setRestrictLimits(int min_min_restrict, int min_max_restrict, int min_def,
                                  int max_min_restrict, int max_max_restrict, int max_def) {
        SpinnerModel spin_model =
        new SpinnerNumberModel(min_def, //initial value
                               min_min_restrict, //min
                               min_max_restrict, //max
                               1);//step

        restrict_spin_min_occur.setModel(spin_model);
        range_min_restrict.setText("(" + min_min_restrict + " - " + min_max_restrict + ")");

        spin_model =
        new SpinnerNumberModel(max_def, //initial value
                               max_min_restrict, //min
                               max_max_restrict, //max
                               1);//step

        restrict_spin_max_occur.setModel(spin_model);
        range_max_restrict.setText("(" + max_min_restrict + " - " + max_max_restrict + ")");
    }

    /**
     * Sets the text with actual value of count of features.
     * @param count an actual value of features count
     */
    public void setActualCountFeatures(int count) {
        this.actual_count_features.setText(Integer.toString(count));
    }

    /**
     * Gets the actual value from the first spinner for minimum occurence.
     * @return value of minimum occurence from the spinner
     */
    public int getRestrictMinOccur() {
        return (Integer) restrict_spin_min_occur.getValue();
    }

    /**
     * Gets the actual value from the second spinner for maximum occurence.
     * @return value of maximum occurence from the spinner
     */
    public int getRestrictMaxOccur() {
        return (Integer) restrict_spin_max_occur.getValue();
    }

    /**
     * Gets the output file format
     * @return output file format
     */
    public int getFileFormat() {
        return select_format.getSelectedIndex();
    }

}
