/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import tp.controllers.MainControl;
import tp.controllers.SettingsControl;

/**
 * Class, which is displaying the panel for pre-processing settings in the application.
 * @author Ondrej Sabatka
 */

public class TabSettingsView extends JPanel {

    //Variables of the class
    private ActionListener action_handler;
    private SettingsControl settings_controller;

    //Spinner used to set maximum size of n-gram
    private JSpinner ngram_spin, ngram_occur;
    private JRadioButton feature_ngram;
    private final JLabel ngram_label;
    private final JLabel ngram_occur_label;
    private JCheckBox load_stop_list_btn;
    private JLabel label_load_stop_list;

    private final JFileChooser fc = new JFileChooser();
    
/**
 * Constructor of the class.
 */
    public TabSettingsView() {
        //Controller setting to process events from GUI
        action_handler = MainControl.getInstance().getSettingsControl();
        settings_controller = MainControl.getInstance().getSettingsControl();

        //Layout settings...
        GridBagConstraints area = new GridBagConstraints();
        this.setLayout(new GridBagLayout());
        area.fill = GridBagConstraints.BOTH;

        //Fonts
        GUIFonts font = new GUIFonts();



        ////////////////////////////////////////////////////////////////////////
        //                     PRE-PROCESSING OPTIONS PANEL                   //
        ////////////////////////////////////////////////////////////////////////
        JPanel preprocess_panel = new JPanel(new GridBagLayout());
        GridBagConstraints preprocess_area = new GridBagConstraints();
        preprocess_panel.setBorder(BorderFactory.createTitledBorder("Options of pre-processing"));

        //Help about format setting...
        preprocess_panel.setName("settings_preprocess.html");
        preprocess_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridx = 0;
        area.gridy = 0;
        area.weightx = 0.5;
        area.weighty = 0.2;

        this.add(preprocess_panel, area);


        preprocess_area.gridx = 0;
        preprocess_area.gridy = 0;
        preprocess_area.weightx = 0.5;
        preprocess_area.weighty = 0.2;
        preprocess_area.gridwidth = 3;
        preprocess_area.anchor = GridBagConstraints.WEST;

        //Componenets on the panel of format setting
        JCheckBox stop_list = new JCheckBox("Stop list - deletion of unimportant words");
        stop_list.setMnemonic(KeyEvent.VK_S);
        stop_list.setSelected(settings_controller.useStopList());
        stop_list.setFont(font.getText());
        preprocess_panel.add(stop_list, preprocess_area);

        preprocess_area.gridy = 1;
        preprocess_area.weightx = 0.2;
        preprocess_area.gridwidth = 1;
        preprocess_panel.add(new JLabel(" "), preprocess_area);

        preprocess_area.gridx = 1;
        preprocess_area.weightx = 0.2;
        load_stop_list_btn = new JCheckBox("Read Stop-list from file");
        load_stop_list_btn.setSelected(settings_controller.useOwnStopList());
        load_stop_list_btn.setFont(font.getText());
        preprocess_panel.add(load_stop_list_btn, preprocess_area);

        //Listener setting
        load_stop_list_btn.setName("ownStopListChange");
        load_stop_list_btn.addActionListener(action_handler);


        preprocess_area.gridx = 2;
        preprocess_area.weightx = 0.6;
        label_load_stop_list = new JLabel("(internal stop-list)");
        label_load_stop_list.setFont(font.getText());
        label_load_stop_list.setForeground(Color.gray);
        preprocess_panel.add(label_load_stop_list, preprocess_area);

        preprocess_area.gridx = 0;
        preprocess_area.gridy = 2;
        preprocess_area.gridwidth = 3;
        
        JCheckBox stemming = new JCheckBox("Stemming - reducing words into their stems     ");
        stemming.setMnemonic(KeyEvent.VK_M);
        stemming.setSelected(settings_controller.useStopList());
        stemming.setFont(font.getText());
        preprocess_panel.add(stemming, preprocess_area);

        //Listener setting
        stop_list.setName("useStopListChange");
        stop_list.addItemListener(settings_controller);

        stemming.setName("useStemmingChange");
        stemming.addItemListener(settings_controller);



        ////////////////////////////////////////////////////////////////////////
        //                     DOCUMENT REPRESENTATION PANEL                  //
        ////////////////////////////////////////////////////////////////////////

        JPanel representation_panel = new JPanel(new GridLayout(10, 1));
        representation_panel.setBorder(BorderFactory.createTitledBorder("Options of document representation"));

        //Help about format setting...
        representation_panel.setName("settings_representation.html");
        representation_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridy = 1;
        area.weighty = 0.4;
        area.fill = GridBagConstraints.BOTH;

        this.add(representation_panel, area);


        //Components on the panel of Loaded documents
        JLabel label = new JLabel("Select feature for document representation: ");
        label.setFont(font.getText());
        representation_panel.add(label);

        int iname = SettingsControl.TypeOfRepresentationFeatures.WORDS.ordinal();

        JRadioButton feature_word = new JRadioButton((SettingsControl.getRepresentationFeatureNames())[iname]);
        feature_word.setMnemonic(KeyEvent.VK_L);
        feature_word.setFont(font.getText());
        feature_word.setName("changeFeature;" + iname);
        feature_word.addActionListener(action_handler);

        if(settings_controller.getRepresentationFeature() == iname) {
            feature_word.setSelected(true);
        }
        else {
            feature_word.setSelected(false);
        }

        /*******************************************************************
         * SMALL PANEL OF N-GRAMS SETTINGS
         ******************************************************************/
        GridBagConstraints ngram_area = new GridBagConstraints();
        ngram_area.fill = GridBagConstraints.BOTH;
        ngram_area.anchor = GridBagConstraints.LINE_START;

        JPanel ngram_panel = new JPanel(new GridBagLayout());

        iname = SettingsControl.TypeOfRepresentationFeatures.NGRAMS.ordinal();

        feature_ngram = new JRadioButton((SettingsControl.getRepresentationFeatureNames())[iname]);
        feature_ngram.setMnemonic(KeyEvent.VK_N);
        feature_ngram.setFont(font.getText());
        feature_ngram.setName("changeFeature;" + iname);
        feature_ngram.addActionListener(action_handler);

        if(settings_controller.getRepresentationFeature() == iname) {
            feature_ngram.setSelected(true);
        }
        else {
            feature_ngram.setSelected(false);
        }

        ngram_area.gridx = 0;
        ngram_area.gridy = 0;
        ngram_area.weightx = 0.1;
        ngram_area.weighty = 0.1;

        ngram_panel.add(feature_ngram, ngram_area);

        ngram_area.gridx = 1;
        ngram_label = new JLabel("  max. size of N-tuple:");
        ngram_label.setFont(font.getText());
        ngram_label.setEnabled(feature_ngram.isSelected());

        ngram_panel.add(ngram_label, ngram_area);

        SpinnerModel spin_model =
        new SpinnerNumberModel(SettingsControl.NGRAM_MIN_DEPTH, //initial value
                               SettingsControl.NGRAM_MIN_DEPTH, //min
                               SettingsControl.NGRAM_MAX_DEPTH, //max
                               1);//step

        ngram_spin = new JSpinner(spin_model);
        ngram_spin.setMinimumSize(new Dimension(30, 5));
        ngram_spin.setEnabled(feature_ngram.isSelected());
        ngram_spin.setName("setNgramDepth");
        ngram_spin.addChangeListener(settings_controller);

        ngram_area.gridx = 2;
        ngram_panel.add(ngram_spin, ngram_area);

        ngram_area.gridx = 3;
        ngram_occur_label = new JLabel("  min. occurence of N-Grams: ");
        ngram_occur_label.setFont(font.getText());
        ngram_occur_label.setEnabled(feature_ngram.isSelected());

        ngram_panel.add(ngram_occur_label, ngram_area);


        spin_model =
        new SpinnerNumberModel(SettingsControl.NGRAM_DEFAULT_OCCUR, //initial value
                               SettingsControl.NGRAM_MIN_OCCUR, //min
                               SettingsControl.NGRAM_MAX_OCCUR, //max
                               1);//step


        ngram_occur = new JSpinner(spin_model);
        ngram_occur.setMinimumSize(new Dimension(30, 5));
        ngram_occur.setEnabled(feature_ngram.isSelected());
        ngram_occur.setName("setNgramOccur");
        ngram_occur.addChangeListener(settings_controller);

        ngram_area.gridx = 4;
        ngram_panel.add(ngram_occur, ngram_area);


        ButtonGroup group_features = new ButtonGroup();
        group_features.add(feature_word);
        group_features.add(feature_ngram);

        representation_panel.add(feature_word);
        representation_panel.add(ngram_panel);


        //Selecting model of representation
        representation_panel.add(new JLabel(""));
        label = new JLabel("Select model for document representation: ");
        label.setFont(font.getText());
        representation_panel.add(label);

        iname = SettingsControl.TypeOfRepresentationModels.BINARY.ordinal();

        JRadioButton model_binary = new JRadioButton((SettingsControl.getRepresentationModelNames())[iname]);
        model_binary.setMnemonic(KeyEvent.VK_B);
        model_binary.setFont(font.getText());
        model_binary.setName("changeModel;" + iname);
        model_binary.addActionListener(action_handler);

        if(settings_controller.getRepresentationModel() == iname) {
            model_binary.setSelected(true);
        }
        else {
            model_binary.setSelected(false);
        }


        iname = SettingsControl.TypeOfRepresentationModels.TF.ordinal();

        JRadioButton model_tf = new JRadioButton((SettingsControl.getRepresentationModelNames())[iname]);
        model_tf.setMnemonic(KeyEvent.VK_T);
        model_tf.setFont(font.getText());
        model_tf.setName("changeModel;" + iname);
        model_tf.addActionListener(action_handler);

        if(settings_controller.getRepresentationModel() == iname) {
            model_tf.setSelected(true);
        }
        else {
            model_tf.setSelected(false);
        }


        iname = SettingsControl.TypeOfRepresentationModels.TFIDF.ordinal();

        JRadioButton model_tfidf = new JRadioButton((SettingsControl.getRepresentationModelNames())[iname]);
        model_tfidf.setMnemonic(KeyEvent.VK_I);
        model_tfidf.setFont(font.getText());
        model_tfidf.setName("changeModel;" + iname);
        model_tfidf.addActionListener(action_handler);

        if(settings_controller.getRepresentationModel() == iname) {
            model_tfidf.setSelected(true);
        }
        else {
            model_tfidf.setSelected(false);
        }


        ButtonGroup group_models = new ButtonGroup();
        group_models.add(model_binary);
        group_models.add(model_tf);
        group_models.add(model_tfidf);

        representation_panel.add(model_binary);
        representation_panel.add(model_tf);
        representation_panel.add(model_tfidf);


        ////////////////////////////////////////////////////////////////////////
        //                            "NEXT" BUTTON                           //
        ////////////////////////////////////////////////////////////////////////
        ImageIcon icon = MainControl.getInstance().getResourceModel().createImageIcon("next.png");
        JButton next_btn = new JButton("Next ", icon);
        next_btn.setFont(font.getTitleText());

        next_btn.setName("nextBtnClick");
        next_btn.addActionListener(action_handler);

        next_btn.setVerticalTextPosition(AbstractButton.CENTER);
        next_btn.setHorizontalTextPosition(AbstractButton.LEADING);

        area.gridy = 2;
        area.weighty = 0.4;
        area.fill = GridBagConstraints.NONE;

        this.add(next_btn, area);
    }


/**
 * Changes the representation feature of a document to n-gram.
 */
    public void changeFeature() {
        ngram_spin.setEnabled(feature_ngram.isSelected());
        ngram_label.setEnabled(feature_ngram.isSelected());
        ngram_occur.setEnabled(feature_ngram.isSelected());
        ngram_occur_label.setEnabled(feature_ngram.isSelected());
    }

/**
 * Gets the maximum size of N-gram.
 * @return maximum size of N-gram
 */
    public int getMaxNgramDepth() {
        return (Integer) ngram_spin.getValue();
    }

/**
 * Gets the minimum occurence of N-gram.
 * @return minimum occurence of N-gram
 */
    public int getMinNgramOccur() {
        return (Integer) ngram_occur.getValue();
    }

/**
 * Changes the usage of a stoplist to false.
 */
    public void isNotUseStopList() {
        load_stop_list_btn.setSelected(false);
        setOwnStopListClear();
        
        label_load_stop_list.setEnabled(false);
        load_stop_list_btn.setEnabled(false);
    }

/**
 * Changes the usage of a stoplist to true.
 */
    public void isUseStopList() {
        label_load_stop_list.setEnabled(true);
        load_stop_list_btn.setEnabled(true);
    }

/**
 * Sets the file name of a user defined stoplist
 * @param name file name of a user defined stoplist
 */
    public void setOwnStopListName(String name) {
        label_load_stop_list.setText(name);
    }

/**
 * Clears the setting of a user defined stoplist, an internal stoplist will be used.
 */
    public void setOwnStopListClear() {
        label_load_stop_list.setText("(internal stop-list)");
    }

 /**
 * Runs the Open file dialog and sets the path to the user defined stoplist.
 * @return file path to the user defined stoplist
 */
    public File getStopListPath() {
        //Run the open dialog..
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile();
        } else {
            load_stop_list_btn.setSelected(false);
            return null;
        }
    }
}
