/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import javax.swing.JTabbedPane;

/**
 * Basic definition of a panel with four tabs
 * @author Ondrej Sabatka
 */

public class TabView extends JTabbedPane {

    //Constants
    public static final int LOADER = 0;
    public static final int SETTINGS = 1;
    public static final int RESULTS = 2;
    public static final int SAVE = 3;
    

    //Variables
    TabLoaderView loader;
    TabSettingsView settings;
    TabResultsView results;
    TabSaveView save;

    /**
     * Constructor of the class.
     */
    public TabView() {
        //Font settings
        this.setFont(new GUIFonts().getTitleText());

        // 1. Read data
        loader = new TabLoaderView();
        this.addTab("1. Read data", loader);

        // 2. Settings
        settings = new TabSettingsView();
        this.addTab("2. Settings", settings);

        // 3. Results overview
        results = new TabResultsView();
        this.addTab("3. Overview", results);

        // 4. Saving
        save = new TabSaveView();
        this.addTab("4. Save", save);

        showTab(LOADER);
    }


    /**
     * Gets the instance of a tab for loading
     * @return TabLoaderView instance
     */
    public TabLoaderView getTabLoader() {
        return loader;
    }

    /**
     * Gets the instance of a tab for settings of preprocessing
     * @return TabSettingsView instance
     */
    public TabSettingsView getTabSettings() {
        return settings;
    }

    /**
     * Gets the instance of a tab for results of preprocessing
     * @return TabResultsView instance
     */
    public TabResultsView getTabResults() {
        return results;
    }

    /**
     * Gets the instance of a tab for saving representations
     * @return TabSaveView instance
     */
    public TabSaveView getTabSave() {
        return save;
    }

    /**
     * Shows the tab with a given identifier.
     * @param id identified of a tab
     */
    public void showTab(int id) {
        switch(id) {
            case SETTINGS:
                 this.setSelectedIndex(id);
                 this.setEnabledAt(id, true);
                 this.setEnabledAt(LOADER, false);
                 this.setEnabledAt(RESULTS, false);
                 this.setEnabledAt(SAVE, false);
                 break;
            case RESULTS:
                 this.setSelectedIndex(id);
                 this.setEnabledAt(id, true);
                 this.setEnabledAt(SETTINGS, false);
                 this.setEnabledAt(LOADER, false);
                 this.setEnabledAt(SAVE, false);
                 break;
            case SAVE:
                 this.setSelectedIndex(id);
                 this.setEnabledAt(id, true);
                 this.setEnabledAt(SETTINGS, false);
                 this.setEnabledAt(LOADER, false);
                 this.setEnabledAt(RESULTS, false);
                 break;
            case LOADER:
            default:
                 this.setSelectedIndex(id);
                 this.setEnabledAt(id, true);
                 this.setEnabledAt(SETTINGS, false);
                 this.setEnabledAt(RESULTS, false);
                 this.setEnabledAt(SAVE, false);
                 break;
        }
        
    }

}
